from pathlib import Path

from django.conf import settings
from django.core.handlers.wsgi import WSGIRequest

static_base = getattr(settings, 'STATIC_PAGE_FILEPATH', './static-pages/')


def get_local_filepath(page):
    site = page.get_site()
    base_path = Path(static_base).joinpath(site.site_name)
    url_path = page.get_url_parts()[2]
    file_path = base_path.joinpath(url_path[1:])
    if not file_path.exists():
        file_path.mkdir(parents=True)
    return file_path.joinpath('index.html')


def create_static_file(page):
    dummy_meta = page._get_dummy_headers()
    dummy_request = WSGIRequest(dummy_meta)
    dummy_request.static_generation = True  # So we don't get served the last static build
    site = page.get_site()
    dummy_request.site = site

    # Pass through serve so we get middleware applied
    response = page.serve(dummy_request)
    response.render()
    page_content = response.content

    path = get_local_filepath(page)
    path.write_bytes(page_content)
    return path


def destroy_all_static_files():
    static_dir = Path(static_base)
    if static_dir.exists():
        for file in static_dir.glob('**/index.html'):
            file.unlink()


def clean_unused_dirs(path=static_base):
    directory = Path(path)
    for child in directory.iterdir():
        if child.is_dir():
            try:
                child.rmdir()
            except OSError:
                clean_unused_dirs(child.absolute())



from wagtail.core.signals import page_published, page_unpublished

from .local_files import create_static_file, get_local_filepath


def static_creation_on_publish(sender, **kwargs):
    page = kwargs['instance']
    if page.is_static:
        create_static_file(page)


page_published.connect(static_creation_on_publish)


def destroy_static_file(sender, **kwargs):
    page = kwargs['instance']
    if page.is_static:
        path = get_local_filepath(page)
        path.unlink(missing_ok=True)


page_unpublished.connect(destroy_static_file)


from pathlib import Path

import boto3
from django.conf import settings

from .local_files import get_local_filepath

static_base = getattr(settings, 'STATIC_PAGE_FILEPATH', './static-pages/')


def upload_site_tree(site):
    client = boto3.client('s3')
    static_dir = Path(static_base, site.site_name)
    for file in static_dir.glob('**/index.html'):
        bucket_path = file.relative_to(static_dir)
        client.upload_file(
            str(file.absolute()),
            'nj-static-test',
            str(bucket_path),
            ExtraArgs={'ContentType': 'text/html'}
        )


def upload_page(page):
    client = boto3.client('s3')
    site = page.get_site()
    static_dir = Path(static_base, site.site_name)
    page_path = get_local_filepath(page)
    bucket_path = page_path.relative_to(static_dir)
    client.upload_file(
        str(page_path.absolute()),
        'nj-static-test',
        str(bucket_path),
        ExtraArgs={'ContentType': 'text/html'}
    )



from functools import wraps

from .local_files import destroy_all_static_files


def static_changer(cls):
    """
    Model decorator, add to models that will change static pages
    """
    def destroy_wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            destroy_all_static_files()
            return f(*args, **kwargs)
        return inner

    # Scorched earth - since we can't know what static file gets affected,
    # just delete all of them
    cls.save = destroy_wrapper(cls.save)
    cls.delete = destroy_wrapper(cls.delete)
    return cls



from django.core.management.base import BaseCommand, CommandError

from wagtail.core.models import Page, Site
from wagtailstatic.s3 import upload_site_tree


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('site_name', type=str)

    def handle(self, *args, **options):
        try:
            site = Site.objects.get(site_name=options['site_name'])
            upload_site_tree(site)
        except Site.DoesNotExist():
            site_names = '\n'.join([s.site_name for s in Site.objects.all()])
            raise CommandError(f'Site not found with that name, options are: {site_names}')
        
        
        


from django.http import HttpResponse
from django.conf import settings
from wagtail.core import hooks

from .local_files import create_static_file, get_local_filepath


@hooks.register('before_serve_page')
def static_serve(page, request, serve_args, serve_kwargs):
    if page.is_static and \
        getattr(settings, 'STATIC_PAGE_SERVING', False) and \
            not getattr(request, 'static_generation', False):
        static_filepath = get_local_filepath(page)
        if not static_filepath.exists():
            static_filepath = create_static_file(page)
        return HttpResponse(static_filepath.read_bytes())



from django.apps import AppConfig


class WagtailStaticConfig(AppConfig):
    name = 'wagtailstatic'
    label = 'wagtailstatic'
    verbose_name = 'Wagtail Static'

    def ready(self):
        import wagtailstatic.signals


#!/usr/bin/env python
"""
Install wagtail-static using setuptools
"""
from setuptools import find_packages, setup

with open('README.rst', 'r') as f:
    readme = f.read()

setup(
    name='wagtail-static',
    version='0.0.2',
    description='Serve pages from filesystem',
    long_description=readme,
    author='Seb Brown',
    author_email='seb@neonjungle.studio',

    install_requires=[
        'wagtail>=2.8',
    ],
    setup_requires=[
        'wheel'
    ],
    zip_safe=False,
    license='BSD License',

    packages=find_packages(exclude=['tests*']),

    include_package_data=True,
    package_data={},

    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
    ],
)



from django.core.management.base import BaseCommand, CommandError
from wagtail.core.models import Page
from wagtailstatic.local_files import (
    clean_unused_dirs, create_static_file, destroy_all_static_files)


class Command(BaseCommand):
    def handle(self, *args, **options):
        destroy_all_static_files()
        for page in Page.objects.all().specific():
            if getattr(page, 'is_static', False):
                self.stdout.write(f'Creating staticfile for: {page.title}')
                create_static_file(page)
        clean_unused_dirs()
